/*
  This code shows how to write a SDL Text Class using an OpenGL window,
  it uses SDL_ttf as a text creator.

  For any problem of installation or usage, please send me a message : 
	http://www.developpez.net/forums/private.php?do=newpm&u=78956

  Copyright (C) 2007 Beyler Jean Christophe

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <SDL.h>
#include <SDL_ttf.h>
#include <math.h>
#include <iostream>

#include "CTexte.h"
using namespace std;

 /* Fonction statique */
static int puissance2sup(int i)
{
    double logbase2 = log(i) / log(2);
    return (int) round(pow(2.0, ceil(logbase2)));
}

/* Methodes de la classe */

/* Constructeur */
CTexte::CTexte()
{
    /* Valeurs par defaut */
    message = "";
    font = NULL;
    font_a_nous = false;
    rasterx = 0;
    rastery = 0;
    textx = 1;
    texty = 1;
    width = 0;
    height = 0;
}

/* Destructeur */
CTexte::~CTexte()
{
    /* Si la police est a nous, on la libere */
    if(font_a_nous) {
        TTF_CloseFont(font);

        glDeleteTextures(1,&surface);
    }
}

/* Chargement d'une police, elle ne sera pas a nous, donc le destructeur ne la liberera pas */
bool CTexte::ChargerPolice(TTF_Font *font, bool anous)
{
    this->font = font;
    font_a_nous = anous;
    return true;
}

/* Chargement d'une police */
bool CTexte::ChargerPolice(string police, int size, int R, int G, int B)
{
	color.r=R;
	color.g=G;
	color.b=B;

	font = TTF_OpenFont(police.c_str(), size);
    if(font==NULL) {
        return false;
    }
    return true;
}

/* Dessiner le texte, gardant le message et la position */
bool CTexte::AffTexte()
{
    if(font != NULL) {
        glEnable(GL_ALPHA_TEST);
        glBlendFunc(GL_ONE,GL_ONE);
        glEnable(GL_BLEND);
        glAlphaFunc(GL_LESS,0.9);
        glEnable(GL_TEXTURE_2D);
        glColor3f(1.0,1.0,1.0);
        glBindTexture(GL_TEXTURE_2D, surface);

        glBegin(GL_QUADS);
            glTexCoord2f (0, 0);
            glVertex2f(rasterx, rastery);

            glTexCoord2f (0, 1);
            glVertex2f(rasterx, rastery + height);

            glTexCoord2f (1, 1);
            glVertex2f(rasterx + width, rastery + height);

            glTexCoord2f (1, 0);
            glVertex2f(rasterx + width, rastery);
        glEnd();
        return true;
    }

    return false;
}

/* Dessiner le texte, gardant le texte */
bool CTexte::AffTexte(int x, int y)
{
    rasterx = x;
    rastery = y;
    return AffTexte();
}

/* Dessiner le texte, gardant la position */
bool CTexte::AffTexte(string message)
{
    if(font != NULL) {
        if(message != this->message) {
            if(this->message != "") {
                glDeleteTextures(1,&surface);
            }

            this->message = message;

            if(message != "") {
                //Création de la texture du sprite
                SDL_Surface *temp;

                glGenTextures (1, & surface);
                glBindTexture (GL_TEXTURE_2D, surface);

                // Paramétrage de la texture.
                glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
                glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
                glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
                glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

                temp = TTF_RenderText_Blended(font, message.c_str(), color);

                if(temp != NULL) {
                    //Calcul de taille
                    int w = puissance2sup(temp->w);
                    int h = puissance2sup(temp->h);

                    SDL_Surface *temp2=SDL_CreateRGBSurface(
                            SDL_HWSURFACE,
                            w,
                            h,
                            32,
                            temp->format->Rmask,
                            temp->format->Gmask,
                            temp->format->Bmask,
                            temp->format->Amask);

                    if(temp2 == NULL) {
                        SDL_FreeSurface(temp);
                        return false;
                    }

                    textx = temp->w;
                    textx /= w;
                    texty = temp->h;
                    texty /= h;
                    width = temp->w;
                    height = temp->h;

                    SDL_BlitSurface(temp, NULL, temp2, NULL);

                    // Jonction entre OpenGL et SDL.
                    glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, temp2->w, temp2->h, 0,
                            GL_RGBA, GL_UNSIGNED_BYTE, temp2->pixels);

                    SDL_FreeSurface(temp);
                    SDL_FreeSurface(temp2);
                }
                else {
                    return false;
                }
            }
        }
    }
    return AffTexte();
}

bool CTexte::AffTexte(string message, int x, int y)
{
    rasterx = x;
    rastery = y;
    return AffTexte(message);
}

void CTexte::SetPosition(int x, int y)
{
	rasterx = x;
	rastery = y;
}

