/*
  This code shows how to write a SDL Text Class using an OpenGL window,
  it uses SDL_ttf as a text creator.

  For any problem of installation or usage, please send me a message : 
	http://www.developpez.net/forums/private.php?do=newpm&u=78956

  Copyright (C) 2007 Beyler Jean Christophe

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include <iostream>
#include <GL/glu.h>
#include <SDL.h>
#include <SDL_framerate.h>

#include "CTexte.h"

using namespace std;
const int WIDTH=800;
const int HEIGHT=600;
static CTexte texte;

void affiche()
{
   //Mettre a zero le tampon de couleurs
   glClear(GL_COLOR_BUFFER_BIT);
   glLoadIdentity();

   //Affichage du texte
   texte.AffTexte();

   SDL_GL_SwapBuffers();
}

int main(int argc, char **argv)
{
    SDL_Event event;
    SDL_Surface *screen;
    int done = 0;
    FPSmanager manage;

    //Initialisation
    if(SDL_Init(SDL_INIT_VIDEO)!=0) {
	   cerr << "Probleme pour initialiser SDL: " << SDL_GetError() << endl;
	   return EXIT_FAILURE;
    }

    if(TTF_Init()==-1) {
        printf("TTF_Init: %s\n", TTF_GetError());
        return EXIT_FAILURE;
    }

    //Mettre un titre à la fenetre
    SDL_WM_SetCaption("Demo Texte", NULL);

    //Initialise le FPSmanager
    SDL_initFramerate(&manage);
    SDL_setFramerate(&manage, 60);

    //Double tampon
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);

    //Ouvrir une fenetre
    screen = SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_OPENGL);
    if(screen==NULL) {
      	done = 1;
    }

    //Preparation du texte
    if(!texte.ChargerPolice("Fabian.ttf", 48, 255, 255, 255)) {
        cerr << "Erreur chargement de la police" << endl;
        return EXIT_FAILURE;
    }
    if(!texte.AffTexte("Test le texte", 0, 0)) {
        cerr << "Erreur affichage du texte" << endl;
        return EXIT_FAILURE;
    }

    //Choisir une couleur
    glClearColor(1.0,0.0,0.0,0.0);

    //Remettre a zero la matrice de projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Mettre les matrices en place pour une vue orthogonale
    gluOrtho2D(0, WIDTH, HEIGHT, 0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    //Boucle generale
    while(!done)
    {
        //Traiter les evenements
        while(SDL_PollEvent(&event))
        {
            switch(event.type)
            {
                case SDL_QUIT:
                    done=1;
                    break;
                case SDL_KEYUP:
                    if(event.key.keysym.sym==SDLK_q) {
                        done=1;
                    }
                    break;
                case SDL_MOUSEMOTION:
                    texte.SetPosition(event.motion.x, event.motion.y);
                    break;
                default:
                    break;
            }
        }

        affiche();
        SDL_framerateDelay(&manage);
    }

    TTF_Quit();
    SDL_Quit();

    (void) argc;
    (void) argv;
    return EXIT_SUCCESS;
}
